@@ -501,7 +501,8 @@ enum {
 	FS_PROP_OVERWRITE,
 	FS_PROP_INTERACTIVE_ONLY,
 	FS_PROP_FORMAT_LEVEL,
-	FS_PROP_SCOPE
+	FS_PROP_SCOPE,
+	FS_PROP_SHEET_SELECTION
 };
 
 enum {
@@ -554,6 +555,9 @@ go_file_saver_set_property (GObject *object, guint property_id,
 	case FS_PROP_SCOPE:
 		fs->save_scope = g_value_get_enum (value);
 		break;
+	case FS_PROP_SHEET_SELECTION:
+		fs->sheet_selection = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -591,6 +595,9 @@ go_file_saver_get_property (GObject *object, guint property_id,
 	case FS_PROP_SCOPE:
 		g_value_set_enum (value, fs->save_scope);
 		break;
+	case FS_PROP_SHEET_SELECTION:
+		g_value_set_boolean (value, fs->sheet_selection);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -702,6 +709,16 @@ go_file_saver_class_init (GOFileSaverClass *klass)
 				    GSF_PARAM_STATIC |
 				    G_PARAM_READWRITE));
 
+        g_object_class_install_property
+		(goc,
+		 FS_PROP_SHEET_SELECTION,
+		 g_param_spec_boolean ("sheet-selection",
+				      _("Sheet Selection"),
+				      _("TRUE if this saver supports saving a subset of all sheet"),
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READWRITE));
+
 	klass->save = go_file_saver_save_real;
 
 	/* class signals */
@@ -1243,16 +1260,28 @@ GOFileSaver *
 go_file_saver_for_file_name (char const *file_name)
 {
 	GList *l;
+	GOFileSaver *best_saver = NULL;
 	char const *extension = gsf_extension_pointer (file_name);
 
-	for (l = default_file_saver_list ; l != NULL; l = l->next)
-		if (!strcmp (go_file_saver_get_extension (((DefaultFileSaver *)(l->data))->saver), extension))
-			return ((DefaultFileSaver *)(l->data))->saver;
+	for (l = default_file_saver_list; l != NULL; l = l->next) {
+		DefaultFileSaver *def_saver = l->data;
+		GOFileSaver *saver = def_saver->saver;
+		if (g_strcmp0 (go_file_saver_get_extension (saver), extension))
+			continue;
+		return saver;
+	}
 
-	for (l = file_saver_list; l != NULL; l = l->next)
-		if (!strcmp (go_file_saver_get_extension (l->data), extension))
-			return l->data;
-	return NULL;
+	for (l = file_saver_list; l != NULL; l = l->next) {
+		GOFileSaver *saver = l->data;
+		if (g_strcmp0 (go_file_saver_get_extension (saver), extension))
+			continue;
+
+		if (!best_saver ||
+		    (saver->save_scope < best_saver->save_scope))
+			best_saver = saver;
+	}
+
+	return best_saver;
 }
 
 /**
