@@ -474,7 +474,7 @@ go_doc_get_image (GODoc *doc, char const *id)
 /**
  * go_doc_add_image:
  * @doc: a #GODoc
- * @id: the image name or NULL
+ * @id: the image name or %NULL
  * @image: a #GOImage
  *
  * Adds @image to the document if no such image already exists. The name of
@@ -595,7 +595,7 @@ go_doc_save_image (GODoc *doc, char const *id)
 	if (!doc)
 		return;
 	if (!g_hash_table_lookup (doc->priv->imagebuf, id)) {
-		GOImage *image = g_hash_table_lookup (doc->images, id);
+		GOImage *image = go_doc_get_image (doc, id);
 		if (image)
 			g_hash_table_replace (doc->priv->imagebuf,
 					      g_strdup (id),
