@@ -556,6 +556,7 @@ gog_view_size_allocate_real (GogView *view, GogViewAllocation const *allocation)
 		if (pos & GOG_POSITION_MANUAL) {
 			available.w = res.w;
 			available.h = res.h;
+
 			gog_view_size_request (child, &available, &req);
 			tmp = gog_object_get_manual_allocation (gog_view_get_model (child),
 								allocation, &req);
@@ -787,9 +788,10 @@ gog_view_padding_request (GogView *view, GogViewAllocation const *bbox, GogViewP
  * @available: available space.
  * @requisition: a #GogViewRequisition.
  *
- * When called @available holds the available space and @requisition is populated
- * with the desired size based on that input and other elements of the view or
- * its model's state (eg the position).
+ * Determines the desired size of a view.
+ *
+ * Note, that the virtual method deviates slightly from this function.  This
+ * function will zero @requisition before calling the virtual method.
  *
  * Remember that the size request is not necessarily the size a view will
  * actually be allocated.
@@ -806,9 +808,10 @@ gog_view_size_request (GogView *view,
 	g_return_if_fail (requisition != NULL);
 	g_return_if_fail (available != NULL);
 
-	if (klass->size_request)
-		(klass->size_request) (view, available, requisition);
-	else
+	if (klass->size_request) {
+		requisition->w = requisition->h = 0;
+		klass->size_request (view, available, requisition);
+	} else
 		requisition->w = requisition->h = 1.;
 }
 
@@ -1049,7 +1052,7 @@ gog_view_get_toolkit (GogView *view)
  * @view: #GogView
  * @x: in coords
  * @y: in coords
- * @gobj: pointed object or NULL
+ * @gobj: pointed object or %NULL
  *
  * Returns: (transfer none): tool under cursor for a given view, or %NULL
  **/
@@ -1080,8 +1083,8 @@ gog_view_get_tool_at_point (GogView *view, double x, double y, GogObject **gobj)
  * @view: #GogView
  * @x: cursor x position
  * @y: cursor y position
- * @obj: pointed object or NULL
- * @tool: pointed tool or NULL
+ * @obj: pointed object or %NULL
+ * @tool: pointed tool or %NULL
  *
  * Gets view under cursor, searching recursively from @view. Corresponding object
  * is stored in @obj. This object may or may not be @view->model of pointed view.
@@ -1130,7 +1133,7 @@ gog_view_get_view_at_point (GogView *view, double x, double y, GogObject **obj,
  * Gets a tip string related to the position as defined by (@x,@y) in @view.
  *
  * return value: the newly allocated tip string if the view class supports
- * that or NULL.
+ * that or %NULL.
  **/
 char*
 gog_view_get_tip_at_point (GogView *view, double x, double y)
